use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::NoTabs 0.15

use Test::More 0.88;
use Test::NoTabs;

my @files = (
    'Changes',
    'GPLv3',
    'MANIFEST.SKIP',
    'VERSION',
    'lib/Systemd/Daemon.pm',
    'lib/Systemd/Daemon/Stub.pm',
    'lib/Systemd/Daemon/XS.pm',
    't/00-stub-export.t',
    't/01-stub-func.t',
    't/02-xs-export.t',
    't/03-xs-func.t',
    't/04-export.t',
    't/05-notify.t',
    't/TestSD.pm',
    't/notify.pl',
    'xt/aspell-en.pws',
    'xt/perlcritic.ini'
);

notabs_ok($_) foreach @files;
done_testing;
